// ---------------------------------------------------------------------------
//
//      The ab-initio based multiscale library
//
//                  S / P H I / n X
//
//      Copyright:  Max-Planck-Institute for Iron Research
//                  40237 Duesseldorf, Germany
//
//      Contact:    https://sxlib.mpie.de
//      Authors:    see sphinx/AUTHORS
//      License:    see sphinx/LICENSE
//
// ---------------------------------------------------------------------------

#ifndef _SX_LOG_BUF_THREAD_H_
#define _SX_LOG_BUF_THREAD_H_

#include <SxIPC.h>
#include <SxLogBuf.h>
#include <SxThreadCond.h>
#include <SxSystemThread.h>

/** \brief Thread safe log bufer

    \b SxLogBufThread = S/PHI/nX Log Buffer Thread

    \author Sixten Boeck, boeck@gemmantics.com
    \author Vaclav Bubnik, bubnik@gemmantics.com */
class SX_EXPORT_IPC SxLogBufThread : public std::streambuf, 
                                     public SxSystemThread
{
   public:
      SxLogBufThread ();
      virtual ~SxLogBufThread ();

      void setFile (const SxString &, int mode=0600); // #exception
 
      void stop ();
      virtual void main ();

   private:
      int overflow (int c);
        
   protected:
      FILE *fp;
      int maxWidth;

      SxArray<char> line;
      ssize_t lineSize;

      SxList<SxString> messages;
      SxString buffer;
      bool stopCmd;
      SxMutex mutex;
      SxThreadCond condition;

      SxString write (const SxString &, bool flush);
};

#endif /* _SX_LOG_BUF_THREAD_H_ */
