// ---------------------------------------------------------------------------
//
//      The ab-initio based multiscale library
//
//                  S / P H I / n X
//
//      Copyright:  Max-Planck-Institute for Iron Research
//                  40237 Duesseldorf, Germany
//
//      Contact:    https://sxlib.mpie.de
//      Authors:    see sphinx/AUTHORS
//      License:    see sphinx/LICENSE
//
// ---------------------------------------------------------------------------

#ifndef _SX_TASK_H_
#define _SX_TASK_H_

#include <SxTimer.h>
#include <SxPtr.h>
#include <SxIPC.h>

class SxProcUnit;

/** \brief ...

    \b SxClass = S/PHI/nX ...

    ....

    \author Sixten Boeck, boeck@mpie.de */
class SX_EXPORT_IPC SxTask
{
   public:

      enum Priority { MaxPriority = 100 }; //  0 - lowest priority
                                           // 99 - highest priority

      SxTask ();

      SxTask (const SxPtr<SxProcUnit> &task_,
              int priority_=0,
              double maxRunTime_=-1.,
              double maxSpoolTime_=-1.);
      ~SxTask ();

      void start ();
      void wait ();

      int getPriority () const { return priority; }
      bool spoolingExpired () const;
      bool runningExpired () const;

      int updatePriority ();

      SxProcUnit *getProcUnitPtr () const { return task.getPtr (); }

   protected:

      enum Timer { Spooling=0, Running=1 };

      int priority;
      double maxRunTime;
      double maxSpoolTime;

      SxTimer timer;

      SxPtr<SxProcUnit> task;

      int prioPoints;

};

#endif /* _SX_TASK_H_ */
