// ---------------------------------------------------------------------------
//
//      The ab-initio based multiscale library
//
//                  S / P H I / n X
//
//      Copyright:  Max-Planck-Institute for Iron Research
//                  40237 Duesseldorf, Germany
//
//      Contact:    https://sxlib.mpie.de
//      Authors:    see sphinx/AUTHORS
//      License:    see sphinx/LICENSE
//
// ---------------------------------------------------------------------------


#include <SxTimeOut.h>


SxTimeOut::SxTimeOut (SxThread *task_, double sec) 
   : SxThread (),
     task (task_),
     timer (1, true),
     timeout (sec),
     exceeded (false)

{
   // empty
}


SxTimeOut::~SxTimeOut ()
{
   // empty
}

void SxTimeOut::main ()
{
   timer.reset ();
   timer.start ();
   task->start ();
   long int interval = 50;  // msec;
   exceeded = false;
   do {
      SxThread::millisleep (interval);
      timer.refresh ();
      if ( timer.getTime () >= timeout)  exceeded = true;
   } while (!exceeded && task->isRunning());
   timer.stop ();
   task->terminate ();
}

bool SxTimeOut::hasExceeded () const
{
   return exceeded;
}
