// ---------------------------------------------------------------------------
//
//      The ab-initio based multiscale library
//
//                  S / P H I / n X
//
//      Copyright:  Max-Planck-Institute for Iron Research
//                  40237 Duesseldorf, Germany
//
//      Contact:    https://sxlib.mpie.de
//      Authors:    see sphinx/AUTHORS
//      License:    see sphinx/LICENSE
//
// ---------------------------------------------------------------------------

#ifndef _SX_MACRO_LIB_H_
#define _SX_MACRO_LIB_H_

#include <SxConfig.h>

// --- Avoid compiler warning of an unused variable
//     usage:
//        void foo (int dummy) {
//           SX_UNUSED(dummy);
//        }
template<class ...Args> void SX_UNUSED(Args&& ... args)
{
    (void)(sizeof...(args));
}

// --------------------------------------------------------------------------
// SX_VMACRO (macroSet, __VA_ARGS__)
//
// Environment to provide variadic macros
// Usage:
//    #define _MyMacro0() ...
//    #define _MyMacro1(a) ...
//    #define _MyMacro2(a,b) ...
//    #define _MyMacro3(a,b,c) ...
//    
//    #define MyMacro(...) SX_VMACRO(_MyMacro, __VA_ARGS__)
//
// Note:    
//    MSVC expands __VA_ARGS__ according to C99. Hence, proper variadic macro
//    unrolling doesn't work:
//    http://connect.microsoft.com/VisualStudio/feedback/details/380090/variadic-macro-replacement
// --------------------------------------------------------------------------
#define _SxVMacroGlue(x,y)  x y
#define _SxVMacroGetNArgs(_1,_2,_3,_4,_5,_6,_7,_8,_9,nArgs,...) nArgs
#define _SxVMacroExpandArgs(args)  _SxVMacroGetNArgs args
#define _SxVMacroCountArgs(...)                                               \
           _SxVMacroExpandArgs((__VA_ARGS__,9,8,7,6,5,4,3,2,1,0))

#define _SxVMacroOvl2(name,nArgs) name##nArgs
#define _SxVMacroOvl1(name,nArgs) _SxVMacroOvl2(name,nArgs)
#define _SxVMacroOvl(name,nArgs)  _SxVMacroOvl1(name,nArgs)

#define SX_VMACRO(name,...)                                                   \
          _SxVMacroGlue(                                                      \
            _SxVMacroOvl(name,_SxVMacroCountArgs(__VA_ARGS__)),               \
            (__VA_ARGS__))






// --------------------------------------------------------------------------
// ATTR_NO_RETURN
//
// ATTR_NO_RETURN indicates that the function never returns
// this removes many false alarms about possibly uninitialized vars
// --------------------------------------------------------------------------
#ifdef ATTR_NO_RETURN
#  error ATTR_NO_RETURN macro name collision - find a better name!
#endif
#ifdef WIN32
#  define ATTR_NO_RETURN
#  define DECL_NO_RETURN __declspec(noreturn)
#else
#  define DECL_NO_RETURN
#  ifdef __GNUC__
#    define ATTR_NO_RETURN __attribute__ ((noreturn))
#  else
#    define ATTR_NO_RETURN
#  endif
#endif


#endif /* _SX_MACRO_LIB_H_ */
