// ---------------------------------------------------------------------------
//
//      The ab-initio based multiscale library
//
//                  S / P H I / n X
//
//      Copyright:  Max-Planck-Institute for Iron Research
//                  40237 Duesseldorf, Germany
//
//      Contact:    https://sxlib.mpie.de
//      Authors:    see sphinx/AUTHORS
//      License:    see sphinx/LICENSE
//
// ---------------------------------------------------------------------------
#ifndef _SX_UTIL_H_
#define _SX_UTIL_H_

#include <math.h>
#include <stdlib.h>
#include <ostream>
#include <SxConfig.h>

#ifdef WIN32
#  if defined(_EXPORT_sxutil)
#     define SX_EXPORT_UTIL __declspec(dllexport)
#  else
#     define SX_EXPORT_UTIL __declspec(dllimport)
#  endif
#else
#  define SX_EXPORT_UTIL
#endif

// --- define unique identifiers
//     Example:  int SX_UNIQUE_ID(abc)  => int abc##__LINE##
#define SX_UNIQUE_ID_NAME(var,x)  var ## x
#define SX_UNIQUE_ID_WRAP(var,x)  SX_UNIQUE_ID_NAME(var,x)
#define SX_UNIQUE_ID(var)         SX_UNIQUE_ID_WRAP(var,__LINE__)

// --- define unit blocks
//     Example: SERIALIZE {
//                 ...
//              }
//              define SX_SERIALIZE  SX_BLOCK(SxMutex)
#define SX_BLOCK(type) \
   bool SX_UNIQUE_ID(cond)=true; for (type SX_UNIQUE_ID(var); SX_UNIQUE_ID(cond); SX_UNIQUE_ID(cond) = false)

#define SX_BLOCK_ARG(type,arg) \
   bool SX_UNIQUE_ID(cond)=true; for (type SX_UNIQUE_ID(var)(arg); SX_UNIQUE_ID(cond); SX_UNIQUE_ID(cond) = false)



class SX_EXPORT_UTIL SxUtil
{
   public:
      SxUtil ();
     ~SxUtil ();

#ifndef SX_IOS
     static SxUtil &getGlobalInstance ();
#endif /* SX_IOS */

     int nProcs;
};

inline int SX_EXPORT_UTIL toInt (float arg)  { return (int)(floor(arg+.5)); }
inline int SX_EXPORT_UTIL toInt (double arg) { return (int)(floor(arg+.5)); }



extern char __flags_;
extern void SX_EXPORT_UTIL setNProcs (int n);
extern int  SX_EXPORT_UTIL getNProcs ();
extern long SX_EXPORT_UTIL sxChunkSize;

extern void SX_EXPORT_UTIL sxGetExecPath (char *res, size_t len); // #exception

extern void SX_EXPORT_UTIL sxchmod (const char *path, int mode); // #exception

#endif /* _SX_UTIL_H_ */
